module Main where
import System.IO (hSetBuffering, BufferMode (NoBuffering), stdout)

main :: IO ()
main =
  hSetBuffering stdout NoBuffering *>
  putStrLn "Hello, Haskell!" *>
  putStr "Please enter your name: " *>
  getLine >>= \name ->
  putStrLn ("Welcome, " ++ name ++ "!") *>
  putStr "Please enter a number: " *>
  readLn >>= \n ->
  print (n :: Int)

-- (*>) :: f a -> f b -> f b

-- A value of type  IO a  is like a *recipe*
-- for an a.  When executed, it can
-- cause things to happen ("effects"), like
-- printing to the screen, reading from a file,
-- sending data over the network, etc.
-- and finally returns a value of type a.
--
-- We can combine recipes into bigger recipes,
-- store them in data structures, etc.
--
-- A value of type IO a is a "first-class imperative program".

-- readLn :: Read a => IO a

-- getLine :: IO String
-- putStrLn :: String -> IO ()

-- (\s _ -> ...) <$> getLine <*> putStrLn "hi"

-- The above pattern works if we know up front
-- exactly what actions we want to do, and
-- just want to combine their results.
-- But it does not give us a way to let
-- later actions depend on the results of
-- previous actions.

-- There is another operator that gives us
-- exactly the ability we want:
--
-- (>>=) :: IO a -> (a -> IO b) -> IO b
--
-- for comparison:
--
-- (*>) :: IO a -> IO b -> IO b
--
-- (>>=) (pronounced "bind") gives us a way
-- to *choose* what action to run second
-- depending on the *output* of the first
-- action.
